//--------------------------------------
// " j o e   p e t s c i i + "
// 256b sprite font exercise 
// by wacek/arise 11/08/2019
//--------------------------------------

.var write_to  		= $a0		
.var main_counter  	= $a4
.var coded_chars 	= $1000

	* = $0900 ">> code (bit-decrunch)"	

generatefont:

//--------------------------------------
// bit-decrunching code
//--------------------------------------	
	
	ldy #0						// byte counter 

decrunch_loop:
	ldx #0						// bit counter

read_bitstream:
	lda bitstream,y				// load the next byte

check_bit:
	and #%10000000				// check the bit
	beq found_zero				// jump if 0
	
	inx							// we have 1, increase bit counter
	jsr next_bit				// move to the next bit
	cpy #61						// did we reach the end of bitstream?
	beq create_charset			// if yes, jump to font creation
	jmp read_bitstream  		// if no, read next bit

found_zero:						// we found zero - end of bitcoded char
write_coded_chars:		
	stx coded_chars				// write depacked char

	inc write_coded_chars + 1	// move byte forward
	lda write_coded_chars + 1	// check if page crossed?
	bne skip_page_crossed		
	inc write_coded_chars + 2	// if crossed, increase hi byte

skip_page_crossed:
	jsr next_bit				// move to the next bit
	jmp decrunch_loop			// rinse & repeat 

//--------------------------------------
// bit-decrunching code - subroutine
//--------------------------------------	

next_bit:	
	ror check_bit + 1			// rotate to the right
	bcc end_next_bit			// if non-zero, then skip to return
	ror check_bit + 1			// if zero, then rotate again
	iny							// that also means we move to next char
end_next_bit:					// in the bitstream
	rts
	
//--------------------------------------
// data - bitstream
//--------------------------------------	

	.memblock ">> data (bitstream)"

bitstream:
	.byte 29,106,184,214,168,106,168,237,106,134	// 26 sprite chars, each described
	.byte 189,80,106,208,215,87,220,107,72,49		// by 9 petscii codes - 234 bytes,
	.byte 3,30,13,251,210,24,168,111,246,144		// primitively bit-crunched to
	.byte 214,144,214,168,107,106,26,213,225,170	// 61 bytes
	.byte 7,245,213,113,165,67,53,67,50,13			//
	.byte 255,106,135,223,189,33,154,188,58,254		// 0 = 0, 10 = 1, 110 = 2 etc.
	.byte 168										//

//--------------------------------------
// sprite - drawing code 
//--------------------------------------	

	.memblock ">> code (draw chars)"

create_charset:
	ldx #$33					// chargen turned on at $d000
	stx $01

	ldx #$20					// set write destination to $2000
	stx write_to + 1			
	ldx #$00					// set depacked chars counter to 0
	stx write_to
	stx main_counter			// set main counter to 0

create_loop:
	jsr do_sprite_line			// "sprite line" meaning 3 petscii 8x8 chars to 8 lines of sprite gfx 
	jsr do_sprite_line			// repeat 3 times
	jsr do_sprite_line			

	clc							// in this routine after each "sprite line" you need to
	lda write_to				// move by 21 bytes, but after the third line you need 
	sbc #7						// to go back a bit
	sta write_to				// this is done in a weird way, but actually saves bytes

	inc main_counter			// move to the next sprite / letter
	lda main_counter			// did we do all 26 chars?
	cmp #26						
	bne not_the_end				// if no, let's continue
	
	lda #255					// we completed sprite drawing, time for some small design fixes
								// this is why the font has "+" in the name ;)
	sta $24a5					// fix "s"
	sta $24a8					//  
	sta $2665					// fix "z"
	sta $2668					//
	sta $2025					// fix "a"
	sta $2319					// fix "m"

the_end:
	nop							// because we have one free byte left... ;)
	rts							// it's all done, jump to viewer

not_the_end:	
	and #%00000011				// here we check if current sprite number is divisible by 4		
	bne create_loop				// if no, we create another sprite
	inc write_to + 1			// if yes, we need to move output to next page
	jmp create_loop				// now, rinse and repeat

//--------------------------------------
// sprite - drawing code - subroutines 
//--------------------------------------	

do_sprite_line:					// here we do one "sprite line" (explained above)
	jsr just_copy				// copy one petscii char
	jsr move_and_copy			// move to next char, copy
	jsr move_and_copy			// move to next char, copy
	inx							
	lda write_to
	adc #21
	sta write_to				// move to next "sprite line"
	rts

//--------------------------------------

move_and_copy:
	inx							// we move to next char for reading
	inc write_to				// and also for writing

just_copy:						
	lda coded_chars,x			// read char (really a table offset, not a petscii code)
	tay					
	lda petscii_address_lo,y	// copy real memory offset address of the petscii char 
	sta read_from + 1
	lda petscii_address_hi,y	
	sta read_from + 2

copy_char:						// copying petscii data to sprite
	ldy #0
read_from:	
	lda $dead					// real memory read adress
	sta (write_to),y			// copy to destination
	iny
	iny							// we increase y by 3 because of how sprite data is layed out
	iny
	inc read_from + 1			// move one byte forward in the chargen
	cpy #24						// did we copy all 8 bytes? 
	bne read_from				// if not, repeat
    rts							// if yes, go back

//--------------------------------------
// data - chargen memory offsets table 
//--------------------------------------	

	.memblock ">> data (chargen offsets)"

petscii_address_hi:
	.byte $d1, $d3, $d5, $d3	// hi bytes of chargen offsets for 10 chars used in sprites 
	.byte $d3, $d2, $d3, $d7
	.byte $d0, $d2

petscii_address_lo:
	.byte $00, $b8, $00, $10	// lo bytes of chargen offsets
	.byte $48, $f8, $78, $78
	.byte $a0, $10

//--------------------------------------
// end of 256b code 
//--------------------------------------	








//--------------------------------------
// viewer code supplied by compo organizer
//--------------------------------------

.pc =$0801 "Basic Starter"
:BasicUpstart($0820)

.pc=$0810 "Constants"
multicol:
.byte $00
bgcol:
.byte $0b
sprcol:
.byte $0f
sprmcol1:
.byte $00
sprmcol2:
.byte $00

.pc = $0820 "Code"
sei
lda multicol
sta $d01c
lda bgcol
sta $d020
sta $d021
lda sprmcol1
sta $d025
lda sprmcol2
sta $d026
lda sprcol
ldx #$07
!loop:	
sta $d027,x
dex
bpl !loop-
ldx #$00
lda #$20
!loop:	
sta $0400,x
sta $0500,x
sta $0600,x
sta $0700,x
inx
bne !loop-
lda #$ff
sta $d015
lda #$80
sta $d010
ldx #$07
ldy #$00
lda #$18+44
!loop:	
sta $d000,y
clc
adc #29
iny
iny
dex
bpl !loop-
jsr generatefont
lda #$35
sta $01
mainloop:
lda #$2f
sta ycoord
lda #$80
sta spridx
yloop:
lda ycoord
!wait:
cmp $d012
bne !wait-	
clc
adc #4
ldx #$07
ldy #$00	
!loop:	
sta $d001,y
iny
iny
dex
bpl !loop-
ldx #$00
ldy spridx	
!loop:	
tya
sta $07f8,x
iny
inx
cpx #8
bne !loop-
clc
lda spridx
adc #8	
sta spridx
lda ycoord
adc #25
sta ycoord
cmp #$2f+8*25
bne yloop
jmp mainloop

ycoord:
.byte 0	
spridx:
.byte 0

